/*
 *  main.c
 *  seeSaw
 *
 *  Created by Daniel Heckenberg.
 *  Copyright (c) 2004 Daniel Heckenberg. All rights reserved.
 *  (danielh.seeSaw<at>cse<dot>unsw<dot>edu<dot>au)
 *  
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the right to use, copy, modify, merge, publish, communicate, sublicence, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included 
 * in all copies or substantial portions of the Software.
 * 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED 
 * "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
 * NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NON-INFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */

#include <GLUT/glut.h>
#include <OpenGL/glext.h>
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "SSUtility.h"
#include "vdigGrab.h"

#include "videoWindow.h"

#include <pthread.h>
pthread_t serverThread;
#include "server.c"

#pragma mark ---- Utilities ----

// Adapted from Apple's GLUT sample
void
printAt(GLfloat x, GLfloat y, const char *string)
{
	int len, i;

	glRasterPos2f(x, y);
	len = strlen(string);
	for (i = 0; i < len; i++) 
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10, string[i]);
}

#pragma mark ---- Main Window data ----

const int MAINWIN_WIDTH = 150;
const int MAINWIN_HEIGHT = 130;

// Main Window data
typedef struct tagMainWindow
{
	int winID;  
	int width;
	int height;
} MainWindow;

// global instance of the main window
MainWindow gMainWindow;

#pragma mark ---- Main Window functions ----

// Adapted from Apple's GLUT sample
void 
drawInfo (GLint window_width, GLint window_height)
{
	char outString [256] = "";
	GLint matrixMode;
	GLint vp[4];
	GLint lineSpacing = 13;
	GLint line = 0;
	GLint startOffest = 7;
	
	glGetIntegerv(GL_VIEWPORT, vp);
	glViewport(0, 0, window_width, window_height);
	
	glGetIntegerv(GL_MATRIX_MODE, &matrixMode);
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	glScalef(2.0f / window_width, -2.0f / window_height, 1.0f);
	glTranslatef(-window_width / 2.0f, -window_height / 2.0f, 0.0f);
	
	// draw 
	glDisable(GL_LIGHTING);
	glColor3f (1.0, 1.0, 1.0);
	
	{
		line = 1;
		sprintf (outString, "Controls:\n");
		printAt (10, (lineSpacing * line++) + startOffest, outString);
		line++;
		sprintf (outString, "c: new camera window\n");
		printAt (10, (lineSpacing * line++) + startOffest, outString);
		sprintf (outString, "x: destroy window\n");
		printAt (10, (lineSpacing * line++) + startOffest, outString);
		sprintf (outString, "f: toggle filter\n");
		printAt (10, (lineSpacing * line++) + startOffest, outString);	
		sprintf (outString, "0: toggle fullscreen\n");
		printAt (10, (lineSpacing * line++) + startOffest, outString);
		sprintf (outString, "1-9: zoom up\n");
		printAt (10, (lineSpacing * line++) + startOffest, outString);
		sprintf (outString, "Alt 1-9: zoom down\n");
		printAt (10, (lineSpacing * line++) + startOffest, outString);
	}
	
	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(matrixMode);
	
	glViewport(vp[0], vp[1], vp[2], vp[3]);
}

void 
mainInitGL(void)     
{
	glShadeModel(GL_FLAT);					
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);				// black window background
}

void 
mainDisplay(void)   
{
	glClearColor(0.5f, 0.5f, 0.5f, 1.0f);				
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	

	drawInfo(gMainWindow.width, gMainWindow.height);

	glutSwapBuffers();
}

void 
mainReshape(int w, int h)    
{
	glViewport(0, 0, w, h);
	
	gMainWindow.width = w;
	gMainWindow.height = h;
  
	glutReshapeWindow(MAINWIN_WIDTH, MAINWIN_HEIGHT); 
}

void
mainDestroyWindow(void)
{
	glutDestroyWindow(glutGetWindow());
}

void mainKeyboard ( unsigned char key, int x, int y )  
{
  switch ( key ) {
	case 'c':		// New camera window
	case 'C':  
		videoNewWindow();
	break;
	case 'x':		// Destroy window
	case 'X':
//		mainDestroyWindow();
	break;
    default:        
      break;
  }
}

void
mainShutdown(void)
{
	// Don't call any GLUT functions here.
	pthread_cancel(serverThread);
	pthread_detach(serverThread);
}

int main (int argc, const char * argv[])  
{
  glutInit(&argc, (char**) argv);
  
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA); 
  glutInitWindowSize(MAINWIN_WIDTH, MAINWIN_HEIGHT); 
  gMainWindow.winID = glutCreateWindow("Help"); 
 
  mainInitGL();
  
  glutDisplayFunc(mainDisplay); 
  glutReshapeFunc(mainReshape);
  glutKeyboardFunc(mainKeyboard);

  // Add a hook so we can shut down nicely
  atexit(mainShutdown);
  
  pthread_create(&serverThread, NULL, serverStart, NULL);
  
  videoNewWindow();
  
  glutMainLoop( );
  
  return 0;
}
